# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/qemu/ui/qemu_vm_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_QemuVMWizard(object):
    def setupUi(self, QemuVMWizard):
        QemuVMWizard.setObjectName("QemuVMWizard")
        QemuVMWizard.resize(647, 452)
        QemuVMWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiServerWizardPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout_5.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.verticalLayout_5.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout_5.addWidget(self.uiLocalRadioButton)
        self.verticalLayout_3.addWidget(self.uiServerTypeGroupBox)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiRemoteServersGroupBox)
        QemuVMWizard.addPage(self.uiServerWizardPage)
        self.uiNameWizardPage = QtWidgets.QWizardPage()
        self.uiNameWizardPage.setObjectName("uiNameWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiNameWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiNameWizardPage)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiNameWizardPage)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        QemuVMWizard.addPage(self.uiNameWizardPage)
        self.uiPlatformMemoryWizardPage = QtWidgets.QWizardPage()
        self.uiPlatformMemoryWizardPage.setObjectName("uiPlatformMemoryWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiPlatformMemoryWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiQemuPlatformLabel = QtWidgets.QLabel(self.uiPlatformMemoryWizardPage)
        self.uiQemuPlatformLabel.setObjectName("uiQemuPlatformLabel")
        self.gridLayout_2.addWidget(self.uiQemuPlatformLabel, 0, 0, 1, 1)
        self.uiQemuPlatformComboBox = QtWidgets.QComboBox(self.uiPlatformMemoryWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiQemuPlatformComboBox.sizePolicy().hasHeightForWidth())
        self.uiQemuPlatformComboBox.setSizePolicy(sizePolicy)
        self.uiQemuPlatformComboBox.setObjectName("uiQemuPlatformComboBox")
        self.gridLayout_2.addWidget(self.uiQemuPlatformComboBox, 0, 1, 1, 1)
        self.uiRamLabel = QtWidgets.QLabel(self.uiPlatformMemoryWizardPage)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_2.addWidget(self.uiRamLabel, 1, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiPlatformMemoryWizardPage)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setProperty("value", 256)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_2.addWidget(self.uiRamSpinBox, 1, 1, 1, 1)
        QemuVMWizard.addPage(self.uiPlatformMemoryWizardPage)
        self.uiConsoleTypeWizardPage = QtWidgets.QWizardPage()
        self.uiConsoleTypeWizardPage.setObjectName("uiConsoleTypeWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiConsoleTypeWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiQemuConsoleTypeComboBox = QtWidgets.QComboBox(self.uiConsoleTypeWizardPage)
        self.uiQemuConsoleTypeComboBox.setObjectName("uiQemuConsoleTypeComboBox")
        self.uiQemuConsoleTypeComboBox.addItem("")
        self.uiQemuConsoleTypeComboBox.addItem("")
        self.uiQemuConsoleTypeComboBox.addItem("")
        self.uiQemuConsoleTypeComboBox.addItem("")
        self.uiQemuConsoleTypeComboBox.addItem("")
        self.verticalLayout.addWidget(self.uiQemuConsoleTypeComboBox)
        self.label = QtWidgets.QLabel(self.uiConsoleTypeWizardPage)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        QemuVMWizard.addPage(self.uiConsoleTypeWizardPage)
        self.uiDiskWizardPage = QtWidgets.QWizardPage()
        self.uiDiskWizardPage.setObjectName("uiDiskWizardPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiDiskWizardPage)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiHdaDiskExistingImageRadioButton = QtWidgets.QRadioButton(self.uiDiskWizardPage)
        self.uiHdaDiskExistingImageRadioButton.setChecked(True)
        self.uiHdaDiskExistingImageRadioButton.setObjectName("uiHdaDiskExistingImageRadioButton")
        self.horizontalLayout_3.addWidget(self.uiHdaDiskExistingImageRadioButton)
        self.uiNewImageRadioButton_2 = QtWidgets.QRadioButton(self.uiDiskWizardPage)
        self.uiNewImageRadioButton_2.setChecked(False)
        self.uiNewImageRadioButton_2.setObjectName("uiNewImageRadioButton_2")
        self.horizontalLayout_3.addWidget(self.uiNewImageRadioButton_2)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.gridLayout_3.addLayout(self.horizontalLayout_3, 0, 0, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.uiHdaDiskImageLabel = QtWidgets.QLabel(self.uiDiskWizardPage)
        self.uiHdaDiskImageLabel.setObjectName("uiHdaDiskImageLabel")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageLabel)
        self.uiHdaDiskImageListComboBox = QtWidgets.QComboBox(self.uiDiskWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiHdaDiskImageListComboBox.sizePolicy().hasHeightForWidth())
        self.uiHdaDiskImageListComboBox.setSizePolicy(sizePolicy)
        self.uiHdaDiskImageListComboBox.setObjectName("uiHdaDiskImageListComboBox")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageListComboBox)
        self.uiHdaDiskImageLineEdit = QtWidgets.QLineEdit(self.uiDiskWizardPage)
        self.uiHdaDiskImageLineEdit.setObjectName("uiHdaDiskImageLineEdit")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageLineEdit)
        self.uiHdaDiskImageToolButton = QtWidgets.QToolButton(self.uiDiskWizardPage)
        self.uiHdaDiskImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiHdaDiskImageToolButton.setObjectName("uiHdaDiskImageToolButton")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageToolButton)
        self.uiHdaDiskImageCreateToolButton = QtWidgets.QToolButton(self.uiDiskWizardPage)
        self.uiHdaDiskImageCreateToolButton.setObjectName("uiHdaDiskImageCreateToolButton")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageCreateToolButton)
        self.gridLayout_3.addLayout(self.horizontalLayout_8, 1, 0, 1, 1)
        QemuVMWizard.addPage(self.uiDiskWizardPage)
        self.uiInitrdKernelImageWizardPage = QtWidgets.QWizardPage()
        self.uiInitrdKernelImageWizardPage.setObjectName("uiInitrdKernelImageWizardPage")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiInitrdKernelImageWizardPage)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiLinuxExistingImageRadioButton = QtWidgets.QRadioButton(self.uiInitrdKernelImageWizardPage)
        self.uiLinuxExistingImageRadioButton.setChecked(True)
        self.uiLinuxExistingImageRadioButton.setObjectName("uiLinuxExistingImageRadioButton")
        self.horizontalLayout_7.addWidget(self.uiLinuxExistingImageRadioButton)
        self.uiNewImageRadioButton_4 = QtWidgets.QRadioButton(self.uiInitrdKernelImageWizardPage)
        self.uiNewImageRadioButton_4.setChecked(False)
        self.uiNewImageRadioButton_4.setObjectName("uiNewImageRadioButton_4")
        self.horizontalLayout_7.addWidget(self.uiNewImageRadioButton_4)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem2)
        self.gridLayout_4.addLayout(self.horizontalLayout_7, 0, 0, 1, 1)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.uiInitrdImageListComboBox = QtWidgets.QComboBox(self.uiInitrdKernelImageWizardPage)
        self.uiInitrdImageListComboBox.setObjectName("uiInitrdImageListComboBox")
        self.horizontalLayout_11.addWidget(self.uiInitrdImageListComboBox)
        self.uiInitrdImageLineEdit = QtWidgets.QLineEdit(self.uiInitrdKernelImageWizardPage)
        self.uiInitrdImageLineEdit.setObjectName("uiInitrdImageLineEdit")
        self.horizontalLayout_11.addWidget(self.uiInitrdImageLineEdit)
        self.uiInitrdImageToolButton = QtWidgets.QToolButton(self.uiInitrdKernelImageWizardPage)
        self.uiInitrdImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiInitrdImageToolButton.setObjectName("uiInitrdImageToolButton")
        self.horizontalLayout_11.addWidget(self.uiInitrdImageToolButton)
        self.formLayout_2.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_11)
        self.uiKernelImageLabel = QtWidgets.QLabel(self.uiInitrdKernelImageWizardPage)
        self.uiKernelImageLabel.setObjectName("uiKernelImageLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.uiKernelImageLabel)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.uiKernelImageListComboBox = QtWidgets.QComboBox(self.uiInitrdKernelImageWizardPage)
        self.uiKernelImageListComboBox.setObjectName("uiKernelImageListComboBox")
        self.horizontalLayout_14.addWidget(self.uiKernelImageListComboBox)
        self.uiKernelImageLineEdit = QtWidgets.QLineEdit(self.uiInitrdKernelImageWizardPage)
        self.uiKernelImageLineEdit.setObjectName("uiKernelImageLineEdit")
        self.horizontalLayout_14.addWidget(self.uiKernelImageLineEdit)
        self.uiKernelImageToolButton = QtWidgets.QToolButton(self.uiInitrdKernelImageWizardPage)
        self.uiKernelImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiKernelImageToolButton.setObjectName("uiKernelImageToolButton")
        self.horizontalLayout_14.addWidget(self.uiKernelImageToolButton)
        self.formLayout_2.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_14)
        self.uiInitrdLabel = QtWidgets.QLabel(self.uiInitrdKernelImageWizardPage)
        self.uiInitrdLabel.setObjectName("uiInitrdLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.uiInitrdLabel)
        self.gridLayout_4.addLayout(self.formLayout_2, 1, 0, 1, 1)
        QemuVMWizard.addPage(self.uiInitrdKernelImageWizardPage)

        self.retranslateUi(QemuVMWizard)
        QtCore.QMetaObject.connectSlotsByName(QemuVMWizard)
        QemuVMWizard.setTabOrder(self.uiNameLineEdit, self.uiQemuPlatformComboBox)
        QemuVMWizard.setTabOrder(self.uiQemuPlatformComboBox, self.uiRamSpinBox)
        QemuVMWizard.setTabOrder(self.uiRamSpinBox, self.uiHdaDiskImageLineEdit)
        QemuVMWizard.setTabOrder(self.uiHdaDiskImageLineEdit, self.uiHdaDiskImageToolButton)

    def retranslateUi(self, QemuVMWizard):
        _translate = QtCore.QCoreApplication.translate
        QemuVMWizard.setWindowTitle(_translate("QemuVMWizard", "New QEMU VM template"))
        self.uiServerWizardPage.setTitle(_translate("QemuVMWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("QemuVMWizard", "Please choose a server type to run the QEMU VM."))
        self.uiServerTypeGroupBox.setTitle(_translate("QemuVMWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("QemuVMWizard", "Run this Qemu VM on a remote computer"))
        self.uiVMRadioButton.setText(_translate("QemuVMWizard", "Run this Qemu VM on the GNS3 VM"))
        self.uiLocalRadioButton.setText(_translate("QemuVMWizard", "Run this Qemu VM on my local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("QemuVMWizard", "Remote server"))
        self.uiRemoteServersLabel.setText(_translate("QemuVMWizard", "Run on:"))
        self.uiNameWizardPage.setTitle(_translate("QemuVMWizard", "QEMU VM name"))
        self.uiNameWizardPage.setSubTitle(_translate("QemuVMWizard", "Please choose a descriptive name for your new QEMU virtual machine."))
        self.uiNameLabel.setText(_translate("QemuVMWizard", "Name:"))
        self.uiPlatformMemoryWizardPage.setTitle(_translate("QemuVMWizard", "QEMU platform and memory"))
        self.uiPlatformMemoryWizardPage.setSubTitle(_translate("QemuVMWizard", "Please select the platform and check the virtual machine has enough memory to work."))
        self.uiQemuPlatformLabel.setText(_translate("QemuVMWizard", "Platform:"))
        self.uiRamLabel.setText(_translate("QemuVMWizard", "RAM:"))
        self.uiRamSpinBox.setSuffix(_translate("QemuVMWizard", " MB"))
        self.uiConsoleTypeWizardPage.setTitle(_translate("QemuVMWizard", "Console type"))
        self.uiConsoleTypeWizardPage.setSubTitle(_translate("QemuVMWizard", "Please choose the console type. Telnet will connect to the serial console of the machine. VNC will connect to graphical output of the machine."))
        self.uiQemuConsoleTypeComboBox.setItemText(0, _translate("QemuVMWizard", "telnet"))
        self.uiQemuConsoleTypeComboBox.setItemText(1, _translate("QemuVMWizard", "vnc"))
        self.uiQemuConsoleTypeComboBox.setItemText(2, _translate("QemuVMWizard", "spice"))
        self.uiQemuConsoleTypeComboBox.setItemText(3, _translate("QemuVMWizard", "spice+agent"))
        self.uiQemuConsoleTypeComboBox.setItemText(4, _translate("QemuVMWizard", "none"))
        self.label.setText(_translate("QemuVMWizard", "Note: You don\'t need to install anything on the VM itself."))
        self.uiDiskWizardPage.setTitle(_translate("QemuVMWizard", "Disk image"))
        self.uiDiskWizardPage.setSubTitle(_translate("QemuVMWizard", "Please choose a base disk image for your virtual machine."))
        self.uiHdaDiskExistingImageRadioButton.setText(_translate("QemuVMWizard", "Existing image"))
        self.uiNewImageRadioButton_2.setText(_translate("QemuVMWizard", "New Image"))
        self.uiHdaDiskImageLabel.setText(_translate("QemuVMWizard", "Disk image (hda):"))
        self.uiHdaDiskImageToolButton.setText(_translate("QemuVMWizard", "&Browse..."))
        self.uiHdaDiskImageCreateToolButton.setText(_translate("QemuVMWizard", "&Create"))
        self.uiInitrdKernelImageWizardPage.setTitle(_translate("QemuVMWizard", "Linux boot specific settings"))
        self.uiInitrdKernelImageWizardPage.setSubTitle(_translate("QemuVMWizard", "Please choose a initrd and a kernel image."))
        self.uiLinuxExistingImageRadioButton.setText(_translate("QemuVMWizard", "Existing image"))
        self.uiNewImageRadioButton_4.setText(_translate("QemuVMWizard", "New Image"))
        self.uiInitrdImageToolButton.setText(_translate("QemuVMWizard", "&Browse..."))
        self.uiKernelImageLabel.setText(_translate("QemuVMWizard", "Kernel image (vmlinuz):"))
        self.uiKernelImageToolButton.setText(_translate("QemuVMWizard", "&Browse..."))
        self.uiInitrdLabel.setText(_translate("QemuVMWizard", "Initial RAM disk (initrd):"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ipaddress


def getNetworkUrl(protocol, host, port, user=None, settings={}):
    """
    Return a network url from settings

    :param protocol: server protocol (http/https)
    :param host: host address
    :param port: port
    :param user: the username
    :param settings: Additional settings
    """

    # 64 character autogenerated user name, we hide it, it's more user friendly
    if user and len(user) != 64:

        try:
            ipaddress.IPv6Address(host.rsplit('%', 1)[0])  # remove any scope ID
            # this is an IPv6 address, we must surround it with brackets to be used in URLs (RFC2732)
            host = "[{}]".format(host)
        except ipaddress.AddressValueError:
            pass

        return "{protocol}://{user}@{host}:{port}".format(protocol=protocol, user=user, host=host, port=port)
    return "{protocol}://{host}:{port}".format(protocol=protocol, host=host, port=port)



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid


class Compute:
    """
    An instance of a compute.
    """

    def __init__(self, compute_id=None):

        if compute_id is None:
            compute_id = str(uuid.uuid4())
        self._compute_id = compute_id
        self._name = compute_id
        self._connected = False
        self._protocol = "http"
        self._host = None
        self._port = 3080
        self._user = None
        self._password = None
        self._cpu_usage_percent = None
        self._memory_usage_percent = None
        self._disk_usage_percent = None
        self._capabilities = {"node_types": []}
        self._last_error = None

    def id(self):
        """
        Returns the compute ID.

        :returns: compute identifier
        """

        return self._compute_id

    def name(self):
        """
        Returns the compute name.

        :returns: compute name
        """

        return self._name

    def setName(self, name):
        """
        Sets the compute name.

        :param name: compute name
        """

        self._name = name

    def connected(self):
        """
        Returns whether or not there is a connection to the compute.

        :returns: boolean
        """

        return self._connected

    def setConnected(self, value):
        """
        Sets whether or not there is a connection to the compute.

        :param value: boolean
        """

        self._connected = value

    def host(self):
        """
        Returns the compute host.

        :returns: host (string)
        """

        return self._host

    def setHost(self, host):
        """
        Sets the compute host.

        :param host: host (string)
        """

        self._host = host

    def port(self):
        """
        Returns the compute port number.

        :returns: port number (integer)
        """

        return self._port

    def setPort(self, port):
        """
        Sets the compute port number.

        :param port: port number (integer)
        """

        self._port = port

    def user(self):
        """
        Returns the compute user for HTTP authentication.

        :returns: user (string)
        """

        return self._user

    def setUser(self, user):
        """
        Sets the compute user for HTTP authentication.

        :param user: user (string)
        """

        self._user = user

    def setPassword(self, password):
        """
        Returns the compute password for HTTP authentication.

        :returns: password (string)
        """

        self._password = password

    def protocol(self):
        """
        Returns the compute protocol.

        :returns: protocol (string)
        """

        return self._protocol

    def setProtocol(self, protocol):
        """
        Sets the compute protocol.

        :param protocol: protocol (string)
        """

        self._protocol = protocol

    def cpuUsagePercent(self):
        """
        Returns the compute CPU usage.

        :returns: CPU usage (integer)
        """

        return self._cpu_usage_percent

    def setCpuUsagePercent(self, usage):
        """
        Sets the compute CPU usage.

        :param usage: CPU usage (integer)
        """

        self._cpu_usage_percent = usage

    def setMemoryUsagePercent(self, usage):
        """
        Returns the compute memory usage.

        :returns: memory usage (integer)
        """

        self._memory_usage_percent = usage

    def memoryUsagePercent(self):
        """
        Sets the compute memory usage.

        :param usage: memory usage (integer)
        """

        return self._memory_usage_percent

    def setDiskUsagePercent(self, usage):
        """
        Sets the compute disk usage.

        :returns: disk usage (integer)
        """

        self._disk_usage_percent = usage

    def diskUsagePercent(self):
        """
        Returns the compute disk usage.

        :param usage: disk usage (integer)
        """

        return self._disk_usage_percent

    def capabilities(self):
        """
        Returns the compute capabilities

        :returns: capabilities (dictionary)
        """

        return self._capabilities

    def setCapabilities(self, value):
        """
        Sets the compute capabilities

        :param value: capabilities (dictionary)
        """

        self._capabilities = value

    def setLastError(self, last_error):
        self._last_error = last_error

    def lastError(self):
        return self._last_error

    def __str__(self):

        return self._compute_id

    def __json__(self):

        return {"host": self._host,
                "port": self._port,
                "protocol": self._protocol,
                "user": self._user,
                "password": self._password,
                "name": self._name,
                "compute_id": self._compute_id}

    def __eq__(self, v):

        if isinstance(v, Compute):
            return self.__json__() == v.__json__()
        return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/appliance_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ApplianceWizard(object):
    def setupUi(self, ApplianceWizard):
        ApplianceWizard.setObjectName("ApplianceWizard")
        ApplianceWizard.resize(726, 428)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ApplianceWizard.sizePolicy().hasHeightForWidth())
        ApplianceWizard.setSizePolicy(sizePolicy)
        ApplianceWizard.setMaximumSize(QtCore.QSize(2000, 2000))
        ApplianceWizard.setModal(True)
        ApplianceWizard.setWizardStyle(QtWidgets.QWizard.ClassicStyle)
        ApplianceWizard.setOptions(QtWidgets.QWizard.NoBackButtonOnStartPage)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiServerWizardPage)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerTypeGroupBox.sizePolicy().hasHeightForWidth())
        self.uiServerTypeGroupBox.setSizePolicy(sizePolicy)
        self.uiServerTypeGroupBox.setMinimumSize(QtCore.QSize(0, 161))
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteRadioButton.sizePolicy().hasHeightForWidth())
        self.uiRemoteRadioButton.setSizePolicy(sizePolicy)
        self.uiRemoteRadioButton.setMinimumSize(QtCore.QSize(0, 20))
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout_2.addWidget(self.uiRemoteRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalRadioButton.sizePolicy().hasHeightForWidth())
        self.uiLocalRadioButton.setSizePolicy(sizePolicy)
        self.uiLocalRadioButton.setMinimumSize(QtCore.QSize(0, 20))
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout_2.addWidget(self.uiLocalRadioButton)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout_4.addWidget(self.uiServerTypeGroupBox)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersGroupBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersGroupBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 1, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiRemoteServersGroupBox)
        ApplianceWizard.addPage(self.uiServerWizardPage)
        self.uiInstructionsPage = QtWidgets.QWizardPage()
        self.uiInstructionsPage.setObjectName("uiInstructionsPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiInstructionsPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiInstructionsTextEdit = QtWidgets.QTextEdit(self.uiInstructionsPage)
        self.uiInstructionsTextEdit.setObjectName("uiInstructionsTextEdit")
        self.verticalLayout_3.addWidget(self.uiInstructionsTextEdit)
        ApplianceWizard.addPage(self.uiInstructionsPage)
        self.uiFilesWizardPage = QtWidgets.QWizardPage()
        self.uiFilesWizardPage.setObjectName("uiFilesWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiFilesWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiApplianceVersionTreeWidget = QtWidgets.QTreeWidget(self.uiFilesWizardPage)
        self.uiApplianceVersionTreeWidget.setIndentation(20)
        self.uiApplianceVersionTreeWidget.setObjectName("uiApplianceVersionTreeWidget")
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.uiApplianceVersionTreeWidget.headerItem().setFont(0, font)
        self.uiApplianceVersionTreeWidget.header().setDefaultSectionSize(120)
        self.uiApplianceVersionTreeWidget.header().setMinimumSectionSize(20)
        self.verticalLayout.addWidget(self.uiApplianceVersionTreeWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiImportPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiImportPushButton.setObjectName("uiImportPushButton")
        self.horizontalLayout.addWidget(self.uiImportPushButton)
        self.uiDownloadPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiDownloadPushButton.setObjectName("uiDownloadPushButton")
        self.horizontalLayout.addWidget(self.uiDownloadPushButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.allowCustomFiles = QtWidgets.QCheckBox(self.uiFilesWizardPage)
        self.allowCustomFiles.setToolTip("")
        self.allowCustomFiles.setObjectName("allowCustomFiles")
        self.horizontalLayout.addWidget(self.allowCustomFiles)
        self.uiCreateVersionPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiCreateVersionPushButton.setObjectName("uiCreateVersionPushButton")
        self.horizontalLayout.addWidget(self.uiCreateVersionPushButton)
        self.uiRefreshPushButton = QtWidgets.QPushButton(self.uiFilesWizardPage)
        self.uiRefreshPushButton.setObjectName("uiRefreshPushButton")
        self.horizontalLayout.addWidget(self.uiRefreshPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        ApplianceWizard.addPage(self.uiFilesWizardPage)
        self.uiUsageWizardPage = QtWidgets.QWizardPage()
        self.uiUsageWizardPage.setObjectName("uiUsageWizardPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiUsageWizardPage)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiUsageTextEdit = QtWidgets.QTextEdit(self.uiUsageWizardPage)
        self.uiUsageTextEdit.setObjectName("uiUsageTextEdit")
        self.gridLayout_3.addWidget(self.uiUsageTextEdit, 0, 0, 1, 1)
        ApplianceWizard.addPage(self.uiUsageWizardPage)

        self.retranslateUi(ApplianceWizard)
        QtCore.QMetaObject.connectSlotsByName(ApplianceWizard)

    def retranslateUi(self, ApplianceWizard):
        _translate = QtCore.QCoreApplication.translate
        ApplianceWizard.setWindowTitle(_translate("ApplianceWizard", "Install appliance"))
        self.uiServerWizardPage.setTitle(_translate("ApplianceWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("ApplianceWizard", "Please choose a server type to install the appliance. The grayed out server types are not supported or configured."))
        self.uiServerTypeGroupBox.setTitle(_translate("ApplianceWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("ApplianceWizard", "Install the appliance on a remote server"))
        self.uiLocalRadioButton.setText(_translate("ApplianceWizard", "Install the appliance on the main server"))
        self.uiRemoteServersGroupBox.setTitle(_translate("ApplianceWizard", "Remote server"))
        self.uiRemoteServersLabel.setText(_translate("ApplianceWizard", "Run on:"))
        self.uiInstructionsPage.setTitle(_translate("ApplianceWizard", "Installation instructions"))
        self.uiInstructionsPage.setSubTitle(_translate("ApplianceWizard", "Please read the following instructions in order to install your new appliance."))
        self.uiInstructionsTextEdit.setHtml(_translate("ApplianceWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.uiFilesWizardPage.setTitle(_translate("ApplianceWizard", "Required files"))
        self.uiFilesWizardPage.setSubTitle(_translate("ApplianceWizard", "The following files are required to install the appliance"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(0, _translate("ApplianceWizard", "Appliance version and files"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(1, _translate("ApplianceWizard", "Size"))
        self.uiApplianceVersionTreeWidget.headerItem().setText(2, _translate("ApplianceWizard", "Status"))
        self.uiImportPushButton.setText(_translate("ApplianceWizard", "&Import"))
        self.uiDownloadPushButton.setText(_translate("ApplianceWizard", "&Download"))
        self.allowCustomFiles.setText(_translate("ApplianceWizard", "Allow custom files"))
        self.uiCreateVersionPushButton.setText(_translate("ApplianceWizard", "&Create a new version"))
        self.uiRefreshPushButton.setText(_translate("ApplianceWizard", "&Refresh"))
        self.uiUsageWizardPage.setTitle(_translate("ApplianceWizard", "Usage"))
        self.uiUsageWizardPage.setSubTitle(_translate("ApplianceWizard", "Please read the following instructions in order to use your new appliance."))
        self.uiUsageTextEdit.setHtml(_translate("ApplianceWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The default username/password is admin/admin. A default configuration is present.</p></body></html>"))
from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from gns3.qt import QtCore, QtGui, QtWidgets
from gns3.ui.show_readme_dialog_ui import Ui_ShowReadmeDialog
from gns3.utils import parse_version

import logging
log = logging.getLogger(__name__)


class ShowReadmeDialog(QtWidgets.QDialog, Ui_ShowReadmeDialog):

    def __init__(self, project, path, content=None, parent=None):

        if parent is None:
            from gns3.main_window import MainWindow
            parent = MainWindow.instance()

        super().__init__(parent)
        self.setupUi(self)

        self._project = project
        self._path = path
        self.setWindowTitle(project.name() + " " + os.path.basename(path))
        self.uiRefreshButton.pressed.connect(self._refreshSlot)

        # QTextDocument before Qt version 5.14 doesn't support Markdown
        if parse_version(QtCore.QT_VERSION_STR) < parse_version("5.14.0") or parse_version(QtCore.PYQT_VERSION_STR) < parse_version("5.14.0"):
            self._markdown = False
        else:
            self._markdown = True

        self._document = QtGui.QTextDocument()
        self.uiTextBrowser.setDocument(self._document)

        if content:
            if self._markdown:
                self._document.setMarkdown(content)
            else:
                self._document.setPlainText(content)
        else:
            self._refreshSlot()

    def _refreshSlot(self):
        self._project.get("/files/" + self._path, self._getCallback, raw=True)

    def _getCallback(self, result, error=False, **kwargs):

        if not error:
            content = result.decode("utf-8", errors="ignore")
            if self._markdown:
                self._document.setMarkdown(content)
            else:
                self._document.setPlainText(content)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import platform
import struct
import distro

try:
    import sentry_sdk
    from sentry_sdk.integrations.logging import LoggingIntegration
    SENTRY_SDK_AVAILABLE = True
except ImportError:
    # Sentry SDK is not installed with deb package in order to simplify packaging
    SENTRY_SDK_AVAILABLE = False

from .version import __version__, __version_info__

import logging
log = logging.getLogger(__name__)


# Dev build
if __version_info__[3] != 0:
    import faulthandler
    # Display a traceback in case of segfault crash. Usefull when frozen
    # Not enabled by default for security reason
    log.debug("Enable catching segfault")
    faulthandler.enable()


class CrashReport:

    """
    Report crash to a third party service
    """

    DSN = "https://f1752c22a8fb159786c8dc37692733d6@o19455.ingest.sentry.io/38506"
    _instance = None

    def __init__(self):
        # We don't want sentry making noise if an error is caught when we don't have internet
        sentry_errors = logging.getLogger('sentry.errors')
        sentry_errors.disabled = True

        sentry_uncaught = logging.getLogger('sentry.errors.uncaught')
        sentry_uncaught.disabled = True
        self._sentry_initialized = False

        if SENTRY_SDK_AVAILABLE:
            # Don't send log records as events.
            sentry_logging = LoggingIntegration(level=logging.INFO, event_level=None)
            try:
                sentry_sdk.init(dsn=CrashReport.DSN,
                                release=__version__,
                                default_integrations=False,
                                integrations=[sentry_logging])
            except Exception as e:
                log.error("Crash report could not be sent: {}".format(e))
                return

            tags = {
                "os:name": platform.system(),
                "os:release": platform.release(),
                "os:win_32": " ".join(platform.win32_ver()),
                "os:mac": "{} {}".format(platform.mac_ver()[0], platform.mac_ver()[2]),
                "os:linux": distro.name(pretty=True),

            }

            self._add_qt_information(tags)

            with sentry_sdk.configure_scope() as scope:
                for key, value in tags.items():
                    scope.set_tag(key, value)

            extra_context = {
                "python:version": "{}.{}.{}".format(sys.version_info[0],
                                                    sys.version_info[1],
                                                    sys.version_info[2]),
                "python:bit": struct.calcsize("P") * 8,
                "python:encoding": sys.getdefaultencoding(),
                "python:frozen": "{}".format(hasattr(sys, "frozen"))
            }

            # extra controller and compute information
            from .controller import Controller
            from .compute_manager import ComputeManager
            extra_context["controller:version"] = Controller.instance().version()
            extra_context["controller:host"] = Controller.instance().host()
            extra_context["controller:connected"] = Controller.instance().connected()

            for index, compute in enumerate(ComputeManager.instance().computes()):
                extra_context["compute{}:id".format(index)] = compute.id()
                extra_context["compute{}:name".format(index)] = compute.name(),
                extra_context["compute{}:host".format(index)] = compute.host(),
                extra_context["compute{}:connected".format(index)] = compute.connected()
                extra_context["compute{}:platform".format(index)] = compute.capabilities().get("platform")
                extra_context["compute{}:version".format(index)] = compute.capabilities().get("version")

            with sentry_sdk.configure_scope() as scope:
                for key, value in extra_context.items():
                    scope.set_extra(key, value)

    def captureException(self, exception, value, tb):

        from .local_server import LocalServer
        from .local_config import LocalConfig

        local_server = LocalServer.instance().localServerSettings()
        if local_server["report_errors"]:

            if not SENTRY_SDK_AVAILABLE:
                log.warning("Cannot capture exception: Sentry SDK is not available")
                return

            if os.path.exists(LocalConfig.instance().runAsRootPath()):
                log.warning("User is running application as root. Crash reports disabled.")
                return

            if not hasattr(sys, "frozen") and os.path.exists(os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", ".git")):
                log.warning(".git directory detected, crash reporting is turned off for developers.")
                return

            try:
                error = (exception, value, tb)
                sentry_sdk.capture_exception(error=error)
                log.info("Crash report sent with event ID: {}".format(sentry_sdk.last_event_id()))
            except Exception as e:
                log.warning("Can't send crash report to Sentry: {}".format(e))

    def _add_qt_information(self, tags):

        try:
            from .qt import QtCore
            from .qt import sip
        except ImportError:
            return tags
        tags["pyqt:version"] = QtCore.PYQT_VERSION_STR
        tags["qt:version"] = QtCore.QT_VERSION_STR
        tags["sip:version"] = sip.SIP_VERSION_STR
        return tags

    @classmethod
    def instance(cls):
        if cls._instance is None:
            cls._instance = CrashReport()
        return cls._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for QEMU VM preferences.
"""

import os
import copy

from gns3.qt import QtCore, QtWidgets, qpartial
from gns3.main_window import MainWindow
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.compute_manager import ComputeManager
from gns3.template_manager import TemplateManager
from gns3.controller import Controller
from gns3.template import Template

from ..settings import QEMU_VM_SETTINGS
from ..ui.qemu_vm_preferences_page_ui import Ui_QemuVMPreferencesPageWidget
from ..pages.qemu_vm_configuration_page import QemuVMConfigurationPage
from ..dialogs.qemu_vm_wizard import QemuVMWizard


class QemuVMPreferencesPage(QtWidgets.QWidget, Ui_QemuVMPreferencesPageWidget):
    """
    QWidget preference page for QEMU VM preferences.
    """

    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._qemu_vms = {}
        self._items = []

        self.uiNewQemuVMPushButton.clicked.connect(self._qemuVMNewSlot)
        self.uiCopyQemuVMPushButton.clicked.connect(self._qemuVMCopySlot)
        self.uiEditQemuVMPushButton.clicked.connect(self._qemuVMEditSlot)
        self.uiDeleteQemuVMPushButton.clicked.connect(self._qemuVMDeleteSlot)
        self.uiQemuVMsTreeWidget.itemSelectionChanged.connect(self._qemuVMChangedSlot)
        self.uiQemuVMsTreeWidget.itemDoubleClicked.connect(self._qemuVMEditSlot)

    def _createSectionItem(self, name):
        """
        Adds a new section to the tree widget.

        :param name: section name
        """

        section_item = QtWidgets.QTreeWidgetItem(self.uiQemuVMInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, qemu_vm):
        """
        Refreshes the content of the tree widget.
        """

        self.uiQemuVMInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtWidgets.QTreeWidgetItem(section_item, ["Template name:", qemu_vm["name"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Template ID:", qemu_vm.get("template_id", "none")])
        if qemu_vm["linked_clone"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Default name format:", qemu_vm["default_name_format"]])
        try:
            compute_id = qemu_vm.get("compute_id")
            if compute_id:
                QtWidgets.QTreeWidgetItem(section_item, ["Compute:", ComputeManager.instance().getCompute(compute_id).name()])
            else:
                if Controller.instance().settings()["dynamic_compute_allocation"]:
                    msg = "Dynamically allocated by the controller"
                else:
                    msg = "Manually chosen"
                QtWidgets.QTreeWidgetItem(section_item, ["Compute:", msg])
        except KeyError:
            pass
        QtWidgets.QTreeWidgetItem(section_item, ["QEMU platform:", os.path.basename(qemu_vm["platform"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Console type:", qemu_vm["console_type"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Auto start console:", "{}".format(qemu_vm["console_auto_start"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Auxiliary console type:", qemu_vm["aux_type"]])
        QtWidgets.QTreeWidgetItem(section_item, ["CPUs:", str(qemu_vm["cpus"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Memory:", "{} MB".format(qemu_vm["ram"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Linked base VM:", "{}".format(qemu_vm["linked_clone"])])

        # fill out the Hard disks section
        if qemu_vm["hda_disk_image"] or qemu_vm["hdb_disk_image"] or qemu_vm["hdc_disk_image"] or qemu_vm["hdd_disk_image"]:
            section_item = self._createSectionItem("Hard disks")
            if qemu_vm["hda_disk_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Disk image (hda):", qemu_vm["hda_disk_image"]])
                QtWidgets.QTreeWidgetItem(section_item, ["Disk interface (hda):", qemu_vm["hda_disk_interface"]])
            if qemu_vm["hdb_disk_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Disk image (hdb):", qemu_vm["hdb_disk_image"]])
                QtWidgets.QTreeWidgetItem(section_item, ["Disk interface (hdb):", qemu_vm["hdb_disk_interface"]])
            if qemu_vm["hdc_disk_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Disk image (hdc):", qemu_vm["hdc_disk_image"]])
                QtWidgets.QTreeWidgetItem(section_item, ["Disk interface (hdc):", qemu_vm["hdc_disk_interface"]])
            if qemu_vm["hdd_disk_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Disk image (hdd):", qemu_vm["hdd_disk_image"]])
                QtWidgets.QTreeWidgetItem(section_item, ["Disk interface (hdd):", qemu_vm["hdd_disk_interface"]])
            if qemu_vm["cdrom_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["CD/DVD image:", qemu_vm["cdrom_image"]])
            if qemu_vm["bios_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Bios image:", qemu_vm["bios_image"]])

        # fill out the Network section
        section_item = self._createSectionItem("Network")
        QtWidgets.QTreeWidgetItem(section_item, ["Adapters:", str(qemu_vm["adapters"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Name format:", qemu_vm["port_name_format"]])
        if qemu_vm["port_segment_size"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Segment size:", str(qemu_vm["port_segment_size"])])
        if qemu_vm["first_port_name"]:
            QtWidgets.QTreeWidgetItem(section_item, ["First port name:", qemu_vm["first_port_name"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Type:", qemu_vm["adapter_type"]])
        if qemu_vm["mac_address"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Base MAC address:", qemu_vm["mac_address"]])

        # fill out the Linux boot section
        if qemu_vm["initrd"] or qemu_vm["kernel_image"] or qemu_vm["kernel_command_line"]:
            section_item = self._createSectionItem("Linux boot")
            if qemu_vm["initrd"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Initial RAM disk:", qemu_vm["initrd"]])
            if qemu_vm["kernel_image"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Kernel image:", qemu_vm["kernel_image"]])
            if qemu_vm["kernel_command_line"]:
                QtWidgets.QTreeWidgetItem(section_item, ["Kernel command line:", qemu_vm["kernel_command_line"]])

        # performance section
        section_item = self._createSectionItem("Optimizations")
        if qemu_vm["cpu_throttling"]:
            QtWidgets.QTreeWidgetItem(section_item, ["CPU throttling:", "{}%".format(qemu_vm["cpu_throttling"])])
        else:
            QtWidgets.QTreeWidgetItem(section_item, ["CPU throttling:", "disabled"])
        QtWidgets.QTreeWidgetItem(section_item, ["Process priority:", qemu_vm["process_priority"]])

        # fill out the Additional options section
        section_item = self._createSectionItem("Additional options")
        if qemu_vm["options"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Options:", qemu_vm["options"]])
        QtWidgets.QTreeWidgetItem(section_item, ["On close:", "{}".format(qemu_vm["on_close"])])

        self.uiQemuVMInfoTreeWidget.expandAll()
        self.uiQemuVMInfoTreeWidget.resizeColumnToContents(0)
        self.uiQemuVMInfoTreeWidget.resizeColumnToContents(1)
        self.uiQemuVMsTreeWidget.setMaximumWidth(self.uiQemuVMsTreeWidget.sizeHintForColumn(0) + 10)

    def _qemuVMChangedSlot(self):
        """
        Loads a selected QEMU VM from the tree widget.
        """

        selection = self.uiQemuVMsTreeWidget.selectedItems()
        self.uiDeleteQemuVMPushButton.setEnabled(len(selection) != 0)
        single_selected = len(selection) == 1
        self.uiEditQemuVMPushButton.setEnabled(single_selected)
        self.uiCopyQemuVMPushButton.setEnabled(single_selected)

        if single_selected:
            key = selection[0].data(0, QtCore.Qt.UserRole)
            qemu_vm = self._qemu_vms[key]
            self._refreshInfo(qemu_vm)
        else:
            self.uiQemuVMInfoTreeWidget.clear()

    def _qemuVMNewSlot(self):
        """
        Creates a new VM.
        """

        wizard = QemuVMWizard(self._qemu_vms, parent=self)
        wizard.show()
        if wizard.exec_():

            new_vm_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=new_vm_settings["compute_id"], name=new_vm_settings["name"])
            if key in self._qemu_vms:
                QtWidgets.QMessageBox.critical(self, "New QEMU VM", "VM name {} already exists".format(new_vm_settings["name"]))
                return
            self._qemu_vms[key] = QEMU_VM_SETTINGS.copy()
            self._qemu_vms[key].update(new_vm_settings)

            item = QtWidgets.QTreeWidgetItem(self.uiQemuVMsTreeWidget)
            item.setText(0, self._qemu_vms[key]["name"])
            Controller.instance().getSymbolIcon(self._qemu_vms[key]["symbol"], qpartial(self._setItemIcon, item))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiQemuVMsTreeWidget.setCurrentItem(item)

    def _qemuVMCopySlot(self):
        """
        Copies a QEMU VM.
        """

        item = self.uiQemuVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            copied_vm_settings = copy.deepcopy(self._qemu_vms[key])
            new_name, ok = QtWidgets.QInputDialog.getText(self, "Copy Qemu template", "Template name:", QtWidgets.QLineEdit.Normal, "Copy of {}".format(copied_vm_settings["name"]))
            if ok:
                key = "{server}:{name}".format(server=copied_vm_settings["compute_id"], name=new_name)
                if key in self._qemu_vms:
                    QtWidgets.QMessageBox.critical(self, "Qemu template", "Qemu template name {} already exists".format(new_name))
                    return
                self._qemu_vms[key] = QEMU_VM_SETTINGS.copy()
                self._qemu_vms[key].update(copied_vm_settings)
                self._qemu_vms[key]["name"] = new_name
                self._qemu_vms[key].pop("template_id", None)

                item = QtWidgets.QTreeWidgetItem(self.uiQemuVMsTreeWidget)
                item.setText(0, self._qemu_vms[key]["name"])
                Controller.instance().getSymbolIcon(self._qemu_vms[key]["symbol"], qpartial(self._setItemIcon, item))
                item.setData(0, QtCore.Qt.UserRole, key)
                self._items.append(item)
                self.uiQemuVMsTreeWidget.setCurrentItem(item)

    def _qemuVMEditSlot(self):
        """
        Edits a QEMU VM.
        """

        item = self.uiQemuVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            qemu_vm = self._qemu_vms[key]
            dialog = ConfigurationDialog(qemu_vm["name"], qemu_vm, QemuVMConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                # update the icon
                Controller.instance().getSymbolIcon(qemu_vm["symbol"], qpartial(self._setItemIcon, item))

                if qemu_vm["name"] != item.text(0):
                    new_key = "{server}:{name}".format(server=qemu_vm["compute_id"], name=qemu_vm["name"])
                    if new_key in self._qemu_vms:
                        QtWidgets.QMessageBox.critical(self, "QEMU VM", "QEMU VM name {} already exists for server {}".format(qemu_vm["name"],
                                                                                                                              qemu_vm["compute_id"]))
                        qemu_vm["name"] = item.text(0)
                        return
                    self._qemu_vms[new_key] = self._qemu_vms[key]
                    del self._qemu_vms[key]
                    item.setText(0, qemu_vm["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)

                self._refreshInfo(qemu_vm)

    def _qemuVMDeleteSlot(self):
        """
        Deletes a QEMU VM.
        """

        for item in self.uiQemuVMsTreeWidget.selectedItems():
            if item:
                key = item.data(0, QtCore.Qt.UserRole)
                del self._qemu_vms[key]
                self.uiQemuVMsTreeWidget.takeTopLevelItem(self.uiQemuVMsTreeWidget.indexOfTopLevelItem(item))

    def loadPreferences(self):
        """
        Loads the QEMU VM preferences.
        """

        self._qemu_vms  = {}
        templates = TemplateManager.instance().templates()
        for template_id, template in templates.items():
            if template.template_type() == "qemu" and not template.builtin():
                name = template.name()
                server = template.compute_id()
                #TODO: use template id for the key
                key = "{server}:{name}".format(server=server, name=name)
                self._qemu_vms[key] = copy.deepcopy(template.settings())

        self._items.clear()
        for key, qemu_vm in self._qemu_vms.items():
            item = QtWidgets.QTreeWidgetItem(self.uiQemuVMsTreeWidget)
            item.setText(0, qemu_vm["name"])
            Controller.instance().getSymbolIcon(qemu_vm["symbol"], qpartial(self._setItemIcon, item))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiQemuVMsTreeWidget.setCurrentItem(self._items[0])
            self.uiQemuVMsTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
            self.uiQemuVMsTreeWidget.setMaximumWidth(self.uiQemuVMsTreeWidget.sizeHintForColumn(0) + 10)

    def _setItemIcon(self, item, icon):
        item.setIcon(0, icon)
        self.uiQemuVMsTreeWidget.setMaximumWidth(self.uiQemuVMsTreeWidget.sizeHintForColumn(0) + 10)

    def savePreferences(self):
        """
        Saves the QEMU VM preferences.
        """

        templates = []
        for template in TemplateManager.instance().templates().values():
            if template.template_type() != "qemu":
                templates.append(template)
        for template_settings in self._qemu_vms.values():
            templates.append(Template(template_settings))
        TemplateManager.instance().updateList(templates)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 Pekka Helenius
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Graphical representation of an Ethernet link for QGraphicsScene.
"""

from ..qt import QtCore, QtGui, QtWidgets
from .link_item import LinkItem
from .label_item import LabelItem
from ..ports.port import Port


class EthernetLinkItem(LinkItem):

    """
    Ethernet link for the scene.

    :param source_item: source NodeItem instance
    :param source_port: source Port instance
    :param destination_item: destination NodeItem instance
    :param destination_port: destination Port instance
    :param link: Link instance (contains back-end stuff for this link)
    :param adding_flag: indicates if this link is being added (no destination yet)
    """

    def __init__(self, source_item, source_port, destination_item, destination_port, link=None, adding_flag=False):

        super().__init__(source_item, source_port, destination_item, destination_port, link, adding_flag)
        self._source_collision_offset = 0.0
        self._destination_collision_offset = 0.0

    def adjust(self):
        """
        Draws a line and compute offsets for status points.
        """

        LinkItem.adjust(self)

        try:
            if self._hovered:
                self.setPen(QtGui.QPen(QtCore.Qt.red, self._link._link_style["width"] + 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
            else:
                self.setPen(QtGui.QPen(QtGui.QColor(self._link._link_style["color"]), self._link._link_style["width"], self._link._link_style["type"], QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        except:
            if self._hovered:
                self.setPen(QtGui.QPen(QtCore.Qt.red, self._pen_width + 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
            else:
                self.setPen(QtGui.QPen(QtGui.QColor("#000000"), self._pen_width, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))

        # draw a line between nodes
        path = QtGui.QPainterPath(self.source)
        path.lineTo(self.destination)
        self.setPath(path)

        # offset on the line for status points
        if self.length == 0:
            self.edge_offset = QtCore.QPointF(0, 0)
        else:
            self.edge_offset = QtCore.QPointF((self.dx * 40) / self.length, (self.dy * 40) / self.length)

    def shape(self):
        """
        Returns the shape of the item to the scene renderer.

        :returns: QPainterPath instance
        """

        path = QtWidgets.QGraphicsPathItem.shape(self)
        offset = self._point_size / 2
        if not self._adding_flag:
            if self.length:
                collision_offset = QtCore.QPointF((self.dx * self._source_collision_offset) / self.length, (self.dy * self._source_collision_offset) / self.length)
            else:
                collision_offset = QtCore.QPointF(0, 0)
            point = self.source + (self.edge_offset + collision_offset)
        else:
            point = self.source
        path.addEllipse(point.x() - offset, point.y() - offset, self._point_size, self._point_size)
        if not self._adding_flag:
            if self.length:
                collision_offset = QtCore.QPointF((self.dx * self._destination_collision_offset) / self.length, (self.dy * self._destination_collision_offset) / self.length)
            else:
                collision_offset = QtCore.QPointF(0, 0)
            point = self.destination - (self.edge_offset + collision_offset)
        else:
            point = self.destination
        path.addEllipse(point.x() - offset, point.y() - offset, self._point_size, self._point_size)
        return path

    def paint(self, painter, option, widget):
        """
        Draws the status points.

        :param painter: QPainter instance
        :param option: QStyleOptionGraphicsItem instance
        :param widget: QWidget instance.
        """

        QtWidgets.QGraphicsPathItem.paint(self, painter, option, widget)
        if not self._adding_flag:

            # points disappears if nodes are too close to each others.
            if self.length < 100:
                return

            if self._link.suspended() or self._source_port.status() == Port.suspended:
                # link or port is suspended
                color = QtCore.Qt.yellow
                shape = QtCore.Qt.RoundCap
            elif self._source_port.status() == Port.started:
                # port is active
                color = QtCore.Qt.green
                shape = QtCore.Qt.RoundCap
            else:
                color = QtCore.Qt.red
                shape = QtCore.Qt.SquareCap

            painter.setPen(QtGui.QPen(color, self._point_size, QtCore.Qt.SolidLine, shape, QtCore.Qt.MiterJoin))
            point1 = QtCore.QPointF(self.source + self.edge_offset) + QtCore.QPointF((self.dx * self._source_collision_offset) / self.length, (self.dy * self._source_collision_offset) / self.length)

            # avoid any collision of the status point with the source node
            while self._source_item.contains(self.mapFromScene(self.mapToItem(self._source_item, point1))):
                self._source_collision_offset += 10
                point1 = QtCore.QPointF(self.source + self.edge_offset) + QtCore.QPointF((self.dx * self._source_collision_offset) / self.length, (self.dy * self._source_collision_offset) / self.length)

            # check with we can paint the status point more closely of the source node
            if not self._source_item.contains(self.mapFromScene(self.mapToItem(self._source_item, point1))):
                check_point = QtCore.QPointF(self.source + self.edge_offset) + QtCore.QPointF((self.dx * (self._source_collision_offset - 20)) / self.length, (self.dy * (self._source_collision_offset - 20)) / self.length)
                if not self._source_item.contains(self.mapFromScene(self.mapToItem(self._source_item, check_point))) and self._source_collision_offset > 0:
                    self._source_collision_offset -= 10

            source_port_label = self._source_port.label()

            if source_port_label is None:
                source_port_label = LabelItem(self._source_item)
                source_port_label.setPlainText(self._source_port.shortName())
                source_port_label.setPos(self.mapToItem(self._source_item, point1))
                self._source_port.setLabel(source_port_label)

            if self._draw_port_labels:
                source_port_label.setFlag(source_port_label.ItemIsMovable, not self._source_item.locked())
                source_port_label.show()
            else:
                source_port_label.hide()

            if self._settings["draw_link_status_points"] and self.pen().style() != QtCore.Qt.NoPen:
                painter.drawPoint(point1)

            if self._link.suspended() or self._destination_port.status() == Port.suspended:
                # link or port is suspended
                color = QtCore.Qt.yellow
                shape = QtCore.Qt.RoundCap
            elif self._destination_port.status() == Port.started:
                # port is active
                color = QtCore.Qt.green
                shape = QtCore.Qt.RoundCap
            else:
                color = QtCore.Qt.red
                shape = QtCore.Qt.SquareCap

            painter.setPen(QtGui.QPen(color, self._point_size, QtCore.Qt.SolidLine, shape, QtCore.Qt.MiterJoin))
            point2 = QtCore.QPointF(self.destination - self.edge_offset) - QtCore.QPointF((self.dx * self._destination_collision_offset) / self.length, (self.dy * self._destination_collision_offset) / self.length)

            # avoid any collision of the status point with the destination node
            while self._destination_item.contains(self.mapFromScene(self.mapToItem(self._destination_item, point2))):
                self._destination_collision_offset += 10
                point2 = QtCore.QPointF(self.destination - self.edge_offset) - QtCore.QPointF((self.dx * self._destination_collision_offset) / self.length, (self.dy * self._destination_collision_offset) / self.length)

            # check with we can paint the status point more closely of the destination node
            if not self._destination_item.contains(self.mapFromScene(self.mapToItem(self._destination_item, point2))):
                check_point = QtCore.QPointF(self.destination - self.edge_offset) - QtCore.QPointF((self.dx * (self._destination_collision_offset - 20)) / self.length, (self.dy * (self._destination_collision_offset - 20)) / self.length)
                if not self._destination_item.contains(self.mapFromScene(self.mapToItem(self._destination_item, check_point))) and self._destination_collision_offset > 0:
                    self._destination_collision_offset -= 10

            destination_port_label = self._destination_port.label()

            if destination_port_label is None:
                destination_port_label = LabelItem(self._destination_item)
                destination_port_label.setPlainText(self._destination_port.shortName())
                destination_port_label.setPos(self.mapToItem(self._destination_item, point2))
                self._destination_port.setLabel(destination_port_label)

            if self._draw_port_labels:
                destination_port_label.setFlag(destination_port_label.ItemIsMovable, not self._destination_item.locked())
                destination_port_label.show()
            else:
                destination_port_label.hide()

            if self._settings["draw_link_status_points"] and self.pen().style() != QtCore.Qt.NoPen:
                painter.drawPoint(point2)

        self._drawSymbol()



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import copy
import os
import collections.abc
import jsonschema
from gns3.utils.get_resource import get_resource


import logging
log = logging.getLogger(__name__)


class ApplianceError(Exception):
    pass


class Appliance(collections.abc.Mapping):

    def __init__(self, registry, path):
        """
        :params registry: Instance of the registry where images are located
        :params path: Path of the appliance file on disk or file content
        """
        self._registry = registry
        self._registry_version = None

        if os.path.isabs(path):
            try:
                with open(path, encoding="utf-8") as f:
                    self._appliance = json.load(f)
            except (OSError, ValueError) as e:
                raise ApplianceError("Could not read appliance {}: {}".format(os.path.abspath(path), str(e)))
        else:
            try:
                self._appliance = json.loads(path)
            except ValueError as e:
                raise ApplianceError("Could not read appliance {}: {}".format(os.path.abspath(path), str(e)))
        self._check_config()
        self._resolve_version()

    def _check_config(self):
        """
        :param appliance: Sanity check on the appliance configuration
        """
        if "registry_version" not in self._appliance:
            raise ApplianceError("Invalid appliance configuration please report the issue on https://github.com/GNS3/gns3-registry/issues")

        self._registry_version = self._appliance["registry_version"]
        if self._registry_version > 8:
            # we only support registry version 8 and below
            raise ApplianceError("Registry version {} is not supported in this version of GNS3".format(self._registry_version))

        if self._registry_version == 8:
            # registry version 8 has a different schema with support for multiple settings sets
            appliance_file = "appliance_v8.json"
        else:
            appliance_file = "appliance.json"

        with open(get_resource("schemas/{}".format(appliance_file))) as f:
            schema = json.load(f)
        v = jsonschema.Draft4Validator(schema)
        try:
            v.validate(self._appliance)
        except jsonschema.ValidationError:
            error = jsonschema.exceptions.best_match(v.iter_errors(self._appliance)).message
            raise ApplianceError("Invalid appliance file: {}".format(error))

    def __getitem__(self, key):
        return self._appliance.__getitem__(key)

    def __iter__(self):
        return self._appliance.__iter__()

    def __len__(self):
        return self._appliance.__len__()

    def _resolve_version(self):
        """
        Replace image field in versions by the complete information from images
        """

        if "versions" not in self._appliance:
            log.debug("No versions found in appliance")
            return

        for version in self._appliance["versions"]:
            for image_type, filename in version["images"].items():

                found = False

                for file in self._appliance["images"]:
                    file = copy.copy(file)

                    if self._registry_version < 8 and "idlepc" in version:
                        file["idlepc"] = version["idlepc"]

                    if "/" in filename:
                        parent = filename.split("/")[0]
                        name = filename.split("/")[-1:][0]
                        filename = os.path.join(parent, name)
                    else:
                        parent = filename

                    if file["filename"] == parent:
                        file["filename"] = filename
                        version["images"][image_type] = file
                        found = True
                        break

                if not found:
                    raise ApplianceError("Broken appliance missing file {} for version {}".format(filename, version["name"]))

    def create_new_version(self, new_version):
        """
        Duplicate a version in order to create a new version
        """

        if "versions" not in self._appliance.keys() or not self._appliance["versions"]:
            raise ApplianceError("Your appliance file doesn't contain any versions")
        self._appliance["versions"].append(new_version)

    def search_images_for_version(self, version_name):
        """
        Search on disk the images required by this version.
        And keep only the required images in the images fields. Add to the images
        their disk type and path.

        :param version_name: Version name
        :returns: Appliance with only require images
        """

        found = False
        appliance = copy.deepcopy(self._appliance)
        for version in appliance["versions"]:
            if version["name"] == version_name:
                appliance["images"] = []
                for image_type, image in version["images"].items():
                    image["type"] = image_type

                    checksum = image.get("md5sum")
                    if checksum is None and self._registry_version >= 8:
                        # registry version >= 8 has the checksum and checksum_type fields
                        checksum_type = image.get("checksum_type", "md5")  # md5 is the default and only supported type
                        if checksum_type != "md5":
                            raise ApplianceError("Checksum type {} is not supported".format(checksum_type))
                        checksum = image.pop("checksum")

                    img = self._registry.search_image_file(self.template_type(), image["filename"], checksum, image.get("filesize"))
                    if img is None:
                        if checksum:
                            raise ApplianceError("File {} with checksum {} not found for {}".format(image["filename"], checksum, appliance["name"]))
                        else:
                            raise ApplianceError("File {} not found for {}".format(image["filename"], appliance["name"]))

                    image["path"] = img.path
                    image["location"] = img.location

                    if "md5sum" not in image:
                        image["md5sum"] = img.md5sum
                        image["filesize"] = img.filesize

                    appliance["images"].append(image)
                    found = True
                # appliance["name"] = "{} {}".format(appliance["name"], version_name)
                break

        if not found:
            raise ApplianceError("Version {} not found for {}".format(version_name, appliance["name"]))

        return appliance

    def copy(self):
        """
        Get a copy of the appliance
        """
        return copy.deepcopy(self._appliance)

    def is_version_installable(self, version):
        """
        Search on disk if a version is available for this appliance

        :params version: Version name
        :returns: Boolean true if installable
        """

        try:
            self.search_images_for_version(version)
            return True
        except ApplianceError:
            return False

    def template_type(self):

        if self._registry_version >= 8:
            template_type = None
            for settings in self._appliance["settings"]:
                if settings["template_type"] and not template_type:
                    template_type = settings["template_type"]
                elif settings["template_type"] and template_type != settings["template_type"]:
                    # we are currently not supporting multiple different template types in the same appliance
                    raise ApplianceError("Multiple different template types found in appliance")
            if not template_type:
                raise ApplianceError("No template type found in appliance {}".format(self._appliance["name"]))
            return template_type
        else:
            if "qemu" in self._appliance:
                return "qemu"
            if "iou" in self._appliance:
                return "iou"
            if "dynamips" in self._appliance:
                return "dynamips"
            if "docker" in self._appliance:
                return "docker"
            return None

    def template_properties(self):
        """
        Get template properties
        """

        if self._registry_version >= 8:
            # find the default settings if any
            for settings in self._appliance["settings"]:
                if settings.get("default", False):
                    return settings["template_properties"]
            # otherwise take the first settings we find
            for settings in self._appliance["settings"]:
                if settings["template_type"]:
                    return settings["template_properties"]
        else:
            if "qemu" in self._appliance:
                return self._appliance["qemu"]
            if "iou" in self._appliance:
                return self._appliance["iou"]
            if "dynamips" in self._appliance:
                return self._appliance["dynamips"]
            if "docker" in self._appliance:
                return self._appliance["docker"]
        return None



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib
from gns3.http_client_error import HttpClientError, HttpClientCancelledRequestError
from ..qt import QtCore, QtWidgets, qslot, sip_is_deleted
from ..ui.image_dialog_ui import Ui_ImageDialog
from ..utils import human_size
from ..controller import Controller


import logging
log = logging.getLogger(__name__)


class ImageDialog(QtWidgets.QDialog, Ui_ImageDialog):
    """
    Image management dialog.
    """

    def __init__(self, parent):
        """
        :param parent: parent widget.
        """
        super().__init__(parent)
        self.setupUi(self)
        self.uiUploadImagePushButton.clicked.connect(self._uploadImageSlot)
        self.uiDeleteImagePushButton.clicked.connect(self._deleteImageSlot)
        self.uiRefreshImagesPushButton.clicked.connect(Controller.instance().refreshImageList)
        Controller.instance().image_list_updated_signal.connect(self._updateImageListSlot)
        self._updateImageListSlot()
        Controller.instance().refreshImageList()

    @qslot
    def _uploadImageSlot(self, *args):

        files, _ = QtWidgets.QFileDialog.getOpenFileNames(
            self,
            "Select one or more images to upload",
            QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DownloadLocation),
            "Images (*.bin *.image *.qcow2 *.vmdk);;All files (*.*)"
        )
        error_msgs = ""
        for path in files:
            log.debug("Uploading image '{}' to controller".format(path))
            image_filename = os.path.basename(path)
            install_appliances = self.uiInstallApplianceCheckBox.isChecked()
            try:
                Controller.instance().post(
                    f"/images/upload/{image_filename}",
                    params={"install_appliances": install_appliances},
                    body=pathlib.Path(path),
                    context={"image_path": path},
                    progress_text="Uploading {}".format(image_filename),
                    timeout=None,
                    wait=True
                )
            except HttpClientCancelledRequestError:
                return
            except HttpClientError as e:
                error_msgs += f"{e}\n"

        if error_msgs:
            error_dialog = QtWidgets.QMessageBox(self)
            error_dialog.setWindowModality(QtCore.Qt.ApplicationModal)
            error_dialog.setWindowTitle("Image upload")
            error_dialog.setText(f"Error while uploading images to the controller")
            error_dialog.setDetailedText(error_msgs)
            error_dialog.setIcon(QtWidgets.QMessageBox.Critical)
            error_dialog.show()

        Controller.instance().refreshImageList()

    @qslot
    def _deleteImageSlot(self, *args):

        if len(self.uiImagesTreeWidget.selectedItems()) == 0:
            QtWidgets.QMessageBox.critical(self, "Delete image", "No images selected")
            return

        reply = QtWidgets.QMessageBox.warning(
            self,
            "Delete image(s)",
            "Delete the selected images?\nThis cannot be reverted.",
            QtWidgets.QMessageBox.Yes,
            QtWidgets.QMessageBox.No
        )

        if reply == QtWidgets.QMessageBox.No:
            return

        images_to_delete = set()
        for image in self.uiImagesTreeWidget.selectedItems():
            if sip_is_deleted(image):
                continue
            image_filename = image.data(0, QtCore.Qt.UserRole)
            images_to_delete.add(image_filename)

        error_msgs = ""
        for image_filename in images_to_delete:
            try:
                Controller.instance().delete(
                    f"/images/{image_filename}",
                    progress_text=f"Deleting {image_filename}",
                    timeout=None,
                    wait=True
                )
            except HttpClientCancelledRequestError:
                return
            except HttpClientError as e:
                error_msgs += f"{e}\n"

        if error_msgs:
            error_dialog = QtWidgets.QMessageBox(self)
            error_dialog.setWindowModality(QtCore.Qt.ApplicationModal)
            error_dialog.setWindowTitle("Image deletion")
            error_dialog.setText(f"Error while deleting images on the controller")
            error_dialog.setDetailedText(error_msgs)
            error_dialog.setIcon(QtWidgets.QMessageBox.Critical)
            error_dialog.show()

        Controller.instance().refreshImageList()

    @qslot
    def _updateImageListSlot(self, *args):

        self.uiImagesTreeWidget.clear()
        self.uiDeleteImagePushButton.setEnabled(False)
        self.uiImagesTreeWidget.setUpdatesEnabled(False)
        items = []
        for image in Controller.instance().images():
            item = QtWidgets.QTreeWidgetItem([image["filename"], image["image_type"], human_size(image["image_size"])])
            item.setData(0, QtCore.Qt.UserRole, image["filename"])
            items.append(item)

        self.uiImagesTreeWidget.addTopLevelItems(items)
        if len(Controller.instance().images()):
            self.uiDeleteImagePushButton.setEnabled(True)

        self.uiImagesTreeWidget.header().setResizeContentsPrecision(100)  # How many rows are checked for the resize for performance reason
        self.uiImagesTreeWidget.resizeColumnToContents(0)
        self.uiImagesTreeWidget.resizeColumnToContents(1)
        self.uiImagesTreeWidget.resizeColumnToContents(2)
        self.uiImagesTreeWidget.sortItems(0, QtCore.Qt.AscendingOrder)
        self.uiImagesTreeWidget.setUpdatesEnabled(True)

    def keyPressEvent(self, e):
        """
        Event handler in order to properly handle escape.
        """

        if e.key() == QtCore.Qt.Key_Escape:
            self.close()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Ethernet switches.
"""

from gns3.qt import QtCore, QtWidgets
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.node import Node

from ..utils.tree_widget_item import TreeWidgetItem
from ..ui.ethernet_switch_configuration_page_ui import Ui_ethernetSwitchConfigPageWidget


class EthernetSwitchConfigurationPage(QtWidgets.QWidget, Ui_ethernetSwitchConfigPageWidget):

    """
    QWidget configuration page for Ethernet switches.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._ports = {}

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

        # connect slots
        self.uiAddPushButton.clicked.connect(self._addPortSlot)
        self.uiDeletePushButton.clicked.connect(self._deletePortSlot)
        self.uiPortsTreeWidget.itemActivated.connect(self._portSelectedSlot)
        self.uiPortsTreeWidget.itemSelectionChanged.connect(self._portSelectionChangedSlot)
        self.uiPortTypeComboBox.currentIndexChanged.connect(self._typeSelectionChangedSlot)

        # enable sorting
        self.uiPortsTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.uiPortsTreeWidget.setSortingEnabled(True)

        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def _portSelectedSlot(self, item, column):
        """
        Loads a selected port from the tree widget.

        :param item: selected TreeWidgetItem instance
        :param column: ignored
        """

        port = int(item.text(0))
        vlan = int(item.text(1))
        port_type = item.text(2)
        port_ethertype = item.text(3)
        self.uiPortSpinBox.setValue(port)
        self.uiVlanSpinBox.setValue(vlan)
        index = self.uiPortTypeComboBox.findText(port_type)
        if index != -1:
            self.uiPortTypeComboBox.setCurrentIndex(index)
        index = self.uiPortEtherTypeComboBox.findText(port_ethertype)
        if index != -1:
            self.uiPortEtherTypeComboBox.setCurrentIndex(index)
        if port_type == "qinq":
            self.uiPortEtherTypeComboBox.setEnabled(True)
        else:
            self.uiPortEtherTypeComboBox.setEnabled(False)

    def _portSelectionChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiPortsTreeWidget.currentItem()
        if item:
            self.uiDeletePushButton.setEnabled(True)
        else:
            self.uiDeletePushButton.setEnabled(False)

    def _typeSelectionChangedSlot(self):
        """
        Disable Q-in-Q EtherType for access and dot1q ports.
        """

        port_type = self.uiPortTypeComboBox.currentText()
        if port_type == "qinq":
            self.uiPortEtherTypeComboBox.setEnabled(True)
        else:
            self.uiPortEtherTypeComboBox.setEnabled(False)

    def _addPortSlot(self):
        """
        Adds a new port.
        """

        port = self.uiPortSpinBox.value()
        vlan = self.uiVlanSpinBox.value()
        port_type = self.uiPortTypeComboBox.currentText()
        if port_type == "qinq":
            port_ethertype = self.uiPortEtherTypeComboBox.currentText()
        else:
            port_ethertype = "0x8100"

        if port in self._ports:
            # update a given entry in the tree widget
            item = self.uiPortsTreeWidget.findItems(str(port), QtCore.Qt.MatchFixedString)[0]
            item.setText(1, str(vlan))
            item.setText(2, port_type)
            if port_ethertype:
                item.setText(3, port_ethertype)

        else:
            # add a new entry in the tree widget
            item = TreeWidgetItem(self.uiPortsTreeWidget)
            item.setText(0, str(port))
            item.setText(1, str(vlan))
            item.setText(2, port_type)
            if port_ethertype:
                item.setText(3, port_ethertype)
            self.uiPortsTreeWidget.addTopLevelItem(item)

        self._ports[port] = {"name": "Ethernet{}".format(port),
                             "port_number": port,
                             "type": port_type,
                             "vlan": vlan,
                             "ethertype": port_ethertype}

        self.uiPortSpinBox.setValue(max(self._ports) + 1)
        self.uiPortsTreeWidget.resizeColumnToContents(0)

    def _deletePortSlot(self):
        """
        Deletes a port.
        """

        item = self.uiPortsTreeWidget.currentItem()
        if item:
            port = int(item.text(0))
            if self._node:
                for node_port in self._node.ports():
                    if node_port.portNumber() == port and not node_port.isFree():
                        QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to port {}, please remove it first".format(node_port.name()))
                        return
            del self._ports[port]
            self.uiPortsTreeWidget.takeTopLevelItem(self.uiPortsTreeWidget.indexOfTopLevelItem(item))

        if len(self._ports):
            self.uiPortSpinBox.setValue(max(self._ports) + 1)
        else:
            self.uiPortSpinBox.setValue(1)

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the Ethernet switch settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
        else:
            self.uiNameLineEdit.setEnabled(False)

        self.uiPortsTreeWidget.clear()
        self._ports = {}
        self._node = node

        if not node:
            # these are template settings

            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            # load the category
            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)
        else:
            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()
            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()
            self.uiCategoryComboBox.hide()

        for port_info in settings["ports_mapping"]:
            item = TreeWidgetItem(self.uiPortsTreeWidget)
            item.setText(0, str(port_info["port_number"]))
            item.setText(1, str(port_info.get("vlan", 1)))
            item.setText(2, port_info.get("type", "access"))
            port_ethertype = port_info.get("ethertype")
            if port_ethertype:
                item.setText(3, port_ethertype)
            self.uiPortsTreeWidget.addTopLevelItem(item)
            self._ports[port_info["port_number"]] = port_info

        # load the console type
        index = self.uiConsoleTypeComboBox.findText(settings["console_type"])
        if index != -1:
            self.uiConsoleTypeComboBox.setCurrentIndex(index)

        self.uiPortsTreeWidget.resizeColumnToContents(0)
        self.uiPortsTreeWidget.resizeColumnToContents(1)
        if len(self._ports) > 0:
            self.uiPortSpinBox.setValue(max(self._ports) + 1)

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the Ethernet switch settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            # set the device name
            name = self.uiNameLineEdit.text()
            if not name:
                QtWidgets.QMessageBox.critical(self, "Name", "Ethernet switch name cannot be empty!")
            else:
                settings["name"] = name

        if not node:
            # these are template settings

            # save the default name format
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            settings["symbol"] = symbol_path

            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())

        # save console type
        settings["console_type"] = self.uiConsoleTypeComboBox.currentText().lower()

        settings["ports_mapping"] = list(self._ports.values())
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import datetime

from gns3.qt import QtCore, QtWidgets, QtGui
from gns3.utils import parse_version
from gns3.http_client_error import HttpClientError, HttpClientCancelledRequestError
from gns3.local_server import LocalServer
from gns3.ui.export_project_wizard_ui import Ui_ExportProjectWizard

import logging
log = logging.getLogger(__name__)


class ExportProjectWizard(QtWidgets.QWizard, Ui_ExportProjectWizard):
    """
    Export project wizard.
    """

    readme_signal = QtCore.pyqtSignal()

    def __init__(self, project, parent):

        super().__init__(parent)
        self.setupUi(self)

        self._project = project
        self._path = None
        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        self.uiCompressionComboBox.addItem("None", "none")
        self.uiCompressionComboBox.addItem("Zip compression (deflate)", "zip")
        self.uiCompressionComboBox.addItem("Bzip2 compression", "bzip2")
        self.uiCompressionComboBox.addItem("Lzma compression", "lzma")
        self.uiCompressionComboBox.addItem("Zstandard compression", "zstd")
        self.uiCompressionComboBox.currentIndexChanged.connect(self._compressionChangedSlot)

        # set zstd compression by default
        self.uiCompressionComboBox.setCurrentIndex(4)
        self.helpRequested.connect(self._showHelpSlot)
        self.uiPathBrowserToolButton.clicked.connect(self._pathBrowserSlot)

        # QTextDocument before Qt version 5.14 doesn't support Markdown
        if parse_version(QtCore.QT_VERSION_STR) < parse_version("5.14.0") or parse_version(QtCore.PYQT_VERSION_STR) < parse_version("5.14.0"):
            self._use_markdown = False
        else:
            self._use_markdown = True

        self._readme_filename = "README.txt"
        self.uiTabWidget.currentChanged.connect(self._previewMarkdownSlot)
        self._loadReadme()

    def _loadReadme(self):

        self._project.get("/files/{}".format(self._readme_filename), self._loadedReadme, raw=True)

    def _loadedReadme(self, result, error=False, context={}, **kwargs):

        if not error:
            content = result.decode("utf-8", errors="replace")
            self.uiReadmeTextEdit.setPlainText(content)
        else:
            if self._use_markdown:
                readme_markdown = "# Project {}\n\nCreated on: {}\n\nAuthor: John Doe <john.doe@example.com>\n\n## Description\n\nNo project description was given".format(self._project.name(), datetime.date.today())
                self.uiReadmeTextEdit.setPlainText(readme_markdown)
            else:
                readme_text = "Project: '{}' created on {}\nAuthor: John Doe <john.doe@example.com>\n\nNo project description was given".format(self._project.name(), datetime.date.today())
                self.uiReadmeTextEdit.setPlainText(readme_text)

    def _pathBrowserSlot(self):

        directory = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DocumentsLocation)
        if len(directory) == 0:
            directory = LocalServer.instance().localServerSettings()["projects_path"]

        path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Export project", directory,
                                                        "GNS3 Project (*.gns3project *.gns3p)",
                                                        "GNS3 Project (*.gns3project *.gns3p)")
        if path is None or len(path) == 0:
            return

        self.uiPathLineEdit.setText(path)

    def _previewMarkdownSlot(self, index):

        # index 1 is preview tab
        if index == 1:

            if self._use_markdown is False:
                QtWidgets.QMessageBox.critical(self, "Markdown preview", "Markdown preview is only support with Qt version 5.14.0 or above")
                return

            # show Markdown preview
            document = QtGui.QTextDocument()
            self.uiReadmePreviewEdit.setDocument(document)
            document.setMarkdown(self.uiReadmeTextEdit.toPlainText())

    def _showHelpSlot(self):

        include_image_help = """Including base images means additional images will not be requested to
        import the project on another computer, however the resulting file will be much bigger.
        Also, you are responsible to check if you have the right to distribute the image(s) as part of the project.
        """
        QtWidgets.QMessageBox.information(self, "Help about export a project", include_image_help)

    def validateCurrentPage(self):
        """
        Validates if the project can be exported.
        """

        if self.currentPage() == self.uiExportOptionsWizardPage:
            path = self.uiPathLineEdit.text().strip()
            if not path:
                QtWidgets.QMessageBox.critical(self, "Export project", "Please select a path where to export the project")
                return False

            if not path.endswith(".gns3project") and not path.endswith(".gns3p"):
                path += ".gns3project"
            try:
                open(path, 'wb+').close()
            except OSError as e:
                QtWidgets.QMessageBox.critical(self, "Export project", "Cannot export project to '{}': {}".format(path, e))
                return False
            self._path = path
        return True

    def _saveReadmeCallback(self, result, error=False, **kwargs):
        if error:
            QtWidgets.QMessageBox.critical(self, "Export project", "Could not created readme file")
        self.readme_signal.emit()

    def waitForReadme(self, signal, timeout=10000):

        # inspired from https://www.jdreaver.com/posts/2014-07-03-waiting-for-signals-pyside-pyqt.html
        loop = QtCore.QEventLoop()
        signal.connect(loop.quit)
        if timeout is not None:
            QtCore.QTimer.singleShot(timeout, loop.quit)
        loop.exec_()

    def _compressionChangedSlot(self, index):
        """
        Set the default compression level.
        """

        compression = self.uiCompressionComboBox.itemData(index)
        self.uiCompressionLevelSpinBox.setEnabled(True)
        if compression == "zip":
            self.uiCompressionLevelSpinBox.setValue(6)  # ZIP default compression level is 6
        elif compression == "bzip2":
            self.uiCompressionLevelSpinBox.setValue(9)  # BZIP2 default compression level is 9
        elif compression == "zstd":
            self.uiCompressionLevelSpinBox.setValue(3)  # ZSTD default compression level is 3
        else:
            # compression level is not supported
            self.uiCompressionLevelSpinBox.setValue(0)
            self.uiCompressionLevelSpinBox.setEnabled(False)

    def done(self, result):
        """
        This dialog is closed.
        """

        if result:

            content = self.uiReadmeTextEdit.toPlainText()
            if content:
                self._project.post("/files/{}".format(self._readme_filename), self._saveReadmeCallback, body=content)

            if self.uiIncludeImagesCheckBox.isChecked():
                include_images = "yes"
            else:
                include_images = "no"
            if self.uiIncludeSnapshotsCheckBox.isChecked():
                include_snapshots = "yes"
            else:
                include_snapshots = "no"
            if self.uiResetMacAddressesCheckBox.isChecked():
                reset_mac_addresses = "yes"
            else:
                reset_mac_addresses = "no"
            compression = self.uiCompressionComboBox.currentData()
            self.waitForReadme(self.readme_signal)

            params = {
                "include_images": include_images,
                "include_snapshots": include_snapshots,
                "reset_mac_addresses": reset_mac_addresses,
                "compression": compression
            }

            try:
                self._project.get(
                    "/export",
                    callback=None,
                    download_progress_callback=self._downloadFileProgress,
                    progress_text="Exporting project files...",
                    params=params,
                    timeout=None,
                    wait=True,
                    raw=True
                )
            except HttpClientCancelledRequestError:
                pass
            except HttpClientError as e:
                QtWidgets.QMessageBox.critical(
                    self,
                    "Project export",
                    f"Could not export project: {e}"
                )

        super().done(result)

    def _downloadFileProgress(self, content, **kwargs):
        """
        Called for each part of the file
        """

        try:
            with open(self._path, 'ab') as f:
                f.write(content)
        except OSError as e:
            log.error(f"Could not write project file: {e}")
            return



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class (interface) for modules.
"""

from ..qt import QtCore
from ..local_config import LocalConfig

import logging
log = logging.getLogger(__name__)


class Module(QtCore.QObject):
    """
    Module interface.
    """

    notification_signal = QtCore.Signal(str, str)

    def __init__(self):

        super().__init__()
        self._settings = {}
        self._nodes = []
        LocalConfig.instance().config_changed_signal.connect(self._configChangedSlot)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def reset(self):
        """
        Resets the module.
        """

        self._nodes.clear()

    def instantiateNode(self, node_class, server, project):
        """
        Instantiate a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        # create an instance of the node class
        return node_class(self, server, project)

    def _configChangedSlot(self):
        """
        Called when the configuration file has changed.
        """

        self._loadSettings()

    def _saveSettings(self):
        """
        Saves the settings to the persistent settings file.
        Must be overloaded.
        """

        raise NotImplementedError()

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        Must be overloaded.
        """

        raise NotImplementedError()

    @staticmethod
    def getNodeClass(node_type, platform=None):
        """
        Returns the class corresponding to node type.
        Must be overloaded.

        :param node_type: name of the node
        :param platform: platform (for Dynamips only)

        :returns: class or None
        """

        raise NotImplementedError()

    @staticmethod
    def preferencePages():
        """
        Returns all the preference pages used by this module.
        Must be overloaded.

        :returns: list of preference page classes
        """

        raise NotImplementedError()

    def exportConfigs(self, directory):
        """
        Exports all configs for all nodes to a directory.

        :param directory: destination directory path
        """

        node_names_cannot_export = []
        for node in self._nodes:
            if hasattr(node, "initialized") and node.initialized():
                if not node.exportConfigsToDirectory(directory):
                    node_names_cannot_export.append(node.name())

        if node_names_cannot_export:
            log.warning("Config export is not supported by the following nodes: {}".format(" ".join(node_names_cannot_export)))

    def importConfigs(self, directory):
        """
        Imports configs to all nodes from a directory.

        :param directory: source directory path
        """

        for node in self._nodes:
            if hasattr(node, "initialized") and node.initialized():
                node.importConfigsFromDirectory(directory)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.node import Node
from .settings import CLOUD_SETTINGS

import logging
log = logging.getLogger(__name__)


class Cloud(Node):

    """
    Cloud node

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    URL_PREFIX = "cloud"

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        # this is an always-on node
        self.setStatus(Node.started)
        self._always_on = True
        self._interfaces = {}
        self._cloud_settings = {"ports_mapping": [],
                                "usage": "",
                                "remote_console_host": CLOUD_SETTINGS["remote_console_host"],
                                "remote_console_port": CLOUD_SETTINGS["remote_console_port"],
                                "remote_console_type": CLOUD_SETTINGS["remote_console_type"],
                                "remote_console_http_path": CLOUD_SETTINGS["remote_console_http_path"]
                                }
        self.settings().update(self._cloud_settings)

    def interfaces(self):

        return self._interfaces

    def _createCallback(self, result):
        """
        Callback for create.

        :param result: server response
        """

        if "interfaces" in result:
            self._interfaces = result["interfaces"].copy()

    def _updateCallback(self, result):
        """
        Callback for update.

        :param result: server response
        """

        if "interfaces" in result:
            self._interfaces = result["interfaces"].copy()

    def consoleType(self):
        """
        Get the console type.
        """

        return self.settings()["remote_console_type"]

    def consoleHost(self):
        """
        Returns the host to connect to the console.

        :returns: host (string)
        """

        return self.settings()["remote_console_host"]

    def console(self):
        """
        Returns the console port number of this node

        :returns: port number
        """

        return self.settings()["remote_console_port"]

    def consoleHttpPath(self):
        """
        Returns the path of the web ui

        :returns: string
        """
        return self._settings["remote_console_http_path"]

    def info(self):
        """
        Returns information about this cloud.

        :returns: formatted string
        """

        info = """Cloud {name} is always-on
  Running on server {host} with port {port}
""".format(name=self.name(),
           host=self.compute().name(),
           port=self.compute().port())

        if self.consoleType() != "none":
            info += """   Remote console is {console_host} on port {console} and type is {console_type}
""".format(console_host=self.consoleHost(),
           console=self.console(),
           console_type=self.consoleType())
            if self.consoleType() in ("http", "https"):
                info += """   Remote console HTTP path is '{console_http_path}'
""".format(console_http_path=self.consoleHttpPath())
        else:
            info += """   No remote console configured
"""

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "   Port {} is empty\n".format(port.name())
            else:
                port_info += "   Port {name} {description}\n".format(name=port.name(),
                                                                     description=port.description())

        usage = "\n" + self._settings.get("usage")
        return info + port_info + usage

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.cloud_configuration_page import CloudConfigurationPage
        return CloudConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this cloud.

        :returns: symbol path (or resource).
        """

        return ":/symbols/cloud.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "Cloud"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Graphical representation of a rectangle on the QGraphicsScene.
"""

import xml.etree.ElementTree as ET

from ..qt import QtCore, QtGui, QtWidgets
from .shape_item import ShapeItem


class RectangleItem(QtWidgets.QGraphicsRectItem, ShapeItem):

    """
    Class to draw a rectangle on the scene.
    """

    def __init__(self, width=200, height=100, **kws):
        self._rx = 0
        self._ry = 0
        super().__init__(width=width, height=height, **kws)

    def setHorizontalCornerRadius(self, radius: int):
        self._rx = radius

    def horizontalCornerRadius(self):
        return self._rx

    def setVerticalCornerRadius(self, radius: int):
        self._ry = radius

    def verticalCornerRadius(self):
        return self._ry

    def paint(self, painter, option, widget=None):
        """
        Paints the contents of an item in local coordinates.

        :param painter: QPainter instance
        :param option: QStyleOptionGraphicsItem instance
        :param widget: QWidget instance
        """

        painter.setPen(self.pen())
        painter.setBrush(self.brush())
        painter.drawRoundedRect(self.rect(), self._rx, self._ry)
        self.drawLayerInfo(painter)

    def toSvg(self):
        """
        Return an SVG version of the shape
        """
        svg = ET.Element("svg")
        svg.set("width", str(int(self.rect().width())))
        svg.set("height", str(int(self.rect().height())))

        rect = ET.SubElement(svg, "rect")
        rect.set("width", str(int(self.rect().width())))
        rect.set("height", str(int(self.rect().height())))
        if self._rx:
            rect.set("rx", str(self._rx))
        if self._ry:
            rect.set("ry", str(self._ry))

        rect = self._styleSvg(rect)

        return ET.tostring(svg, encoding="utf-8").decode("utf-8")

    def fromSvg(self, svg):
        svg_elem = ET.fromstring(svg)
        if len(svg_elem):
            # handle horizontal corner radius and vertical corner radius (specific to rectangles)
            rx = svg_elem[0].get("rx")
            ry = svg_elem[0].get("ry")
            if rx:
                self._rx = int(rx)
            elif ry:
                self._rx = int(ry)  # defaults to ry if it is specified
            if ry:
                self._ry = int(ry)
            elif rx:
                self._ry = int(rx)  # defaults to rx if it is specified
        super().fromSvg(svg)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for IOU device preferences.
"""

import copy
import os

from gns3.qt import QtCore, QtWidgets, qpartial

from gns3.main_window import MainWindow
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.image_manager import ImageManager
from gns3.compute_manager import ComputeManager
from gns3.template_manager import TemplateManager
from gns3.controller import Controller
from gns3.template import Template

from ..settings import IOU_DEVICE_SETTINGS
from ..ui.iou_device_preferences_page_ui import Ui_IOUDevicePreferencesPageWidget
from ..pages.iou_device_configuration_page import iouDeviceConfigurationPage
from ..dialogs.iou_device_wizard import IOUDeviceWizard


class IOUDevicePreferencesPage(QtWidgets.QWidget, Ui_IOUDevicePreferencesPageWidget):

    """
    QWidget preference page for IOU image & device preferences.
    """

    _default_images_dir = ""

    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._iou_devices = {}
        self._items = []

        self.uiNewIOUDevicePushButton.clicked.connect(self._iouDeviceNewSlot)
        self.uiCopyIOUDevicePushButton.clicked.connect(self._iouDeviceCopySlot)
        self.uiEditIOUDevicePushButton.clicked.connect(self._iouDeviceEditSlot)
        self.uiDeleteIOUDevicePushButton.clicked.connect(self._iouDeviceDeleteSlot)
        self.uiIOUDevicesTreeWidget.itemSelectionChanged.connect(self._iouDeviceChangedSlot)
        self.uiIOUDevicesTreeWidget.itemDoubleClicked.connect(self._iouDeviceEditSlot)

    def _createSectionItem(self, name):
        """
        Adds a new section to the tree widget.

        :param name: section name
        """

        section_item = QtWidgets.QTreeWidgetItem(self.uiIOUDeviceInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, iou_device):
        """
        Refreshes the content of the tree widget.
        """

        self.uiIOUDeviceInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtWidgets.QTreeWidgetItem(section_item, ["Template name:", iou_device["name"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Template ID:", iou_device.get("template_id", "none")])
        QtWidgets.QTreeWidgetItem(section_item, ["Default name format:", iou_device["default_name_format"]])
        try:
            compute_id = iou_device.get("compute_id")
            if compute_id:
                QtWidgets.QTreeWidgetItem(section_item, ["Compute:", ComputeManager.instance().getCompute(compute_id).name()])
            else:
                if Controller.instance().settings()["dynamic_compute_allocation"]:
                    msg = "Dynamically allocated by the controller"
                else:
                    msg = "Manually chosen"
                QtWidgets.QTreeWidgetItem(section_item, ["Compute:", msg])
        except KeyError:
            # Compute doesn't exists
            pass
        QtWidgets.QTreeWidgetItem(section_item, ["Console type:", iou_device["console_type"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Auto start console:", "{}".format(iou_device["console_auto_start"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Image:", iou_device["path"]])
        if iou_device["startup_config"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Startup-config:", iou_device["startup_config"]])

        if iou_device["private_config"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Private-config:", iou_device["private_config"]])

        if iou_device["use_default_iou_values"]:
            QtWidgets.QTreeWidgetItem(section_item, ["RAM:", "default"])
            QtWidgets.QTreeWidgetItem(section_item, ["NVRAM:", "default"])
        else:
            QtWidgets.QTreeWidgetItem(section_item, ["RAM:", "{} MiB".format(iou_device["ram"])])
            QtWidgets.QTreeWidgetItem(section_item, ["NVRAM:", "{} KiB".format(iou_device["nvram"])])

        # fill out the Network section
        section_item = self._createSectionItem("Network")
        QtWidgets.QTreeWidgetItem(section_item, ["Ethernet adapters:", "{} ({} interfaces)".format(iou_device["ethernet_adapters"],
                                                                                                   iou_device["ethernet_adapters"] * 4)])
        QtWidgets.QTreeWidgetItem(section_item, ["Serial adapters:", "{} ({} interfaces)".format(iou_device["serial_adapters"],
                                                                                                 iou_device["serial_adapters"] * 4)])

        self.uiIOUDeviceInfoTreeWidget.expandAll()
        self.uiIOUDeviceInfoTreeWidget.resizeColumnToContents(0)
        self.uiIOUDeviceInfoTreeWidget.resizeColumnToContents(1)
        self.uiIOUDevicesTreeWidget.setMaximumWidth(self.uiIOUDevicesTreeWidget.sizeHintForColumn(0) + 10)

    def _iouDeviceChangedSlot(self):
        """
        Loads a selected an IOU device from the tree widget.
        """

        selection = self.uiIOUDevicesTreeWidget.selectedItems()
        self.uiDeleteIOUDevicePushButton.setEnabled(len(selection) != 0)
        single_selected = len(selection) == 1
        self.uiEditIOUDevicePushButton.setEnabled(single_selected)
        self.uiCopyIOUDevicePushButton.setEnabled(single_selected)

        if single_selected:
            key = selection[0].data(0, QtCore.Qt.UserRole)
            iou_device = self._iou_devices[key]
            self._refreshInfo(iou_device)
        else:
            self.uiIOUDeviceInfoTreeWidget.clear()

    def _iouDeviceNewSlot(self):
        """
        Creates a new IOU device.
        """

        wizard = IOUDeviceWizard(self._iou_devices, parent=self)
        wizard.show()
        if wizard.exec_():

            new_device_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=new_device_settings["compute_id"], name=new_device_settings["name"])
            self._iou_devices[key] = IOU_DEVICE_SETTINGS.copy()
            self._iou_devices[key].update(new_device_settings)

            item = QtWidgets.QTreeWidgetItem(self.uiIOUDevicesTreeWidget)
            item.setText(0, self._iou_devices[key]["name"])
            Controller.instance().getSymbolIcon(self._iou_devices[key]["symbol"], qpartial(self._setItemIcon, item))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiIOUDevicesTreeWidget.setCurrentItem(item)

    def _iouDeviceCopySlot(self):
        """
        Copies an IOU device.
        """

        item = self.uiIOUDevicesTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            copied_iou_device_settings = copy.deepcopy(self._iou_devices[key])
            new_name, ok = QtWidgets.QInputDialog.getText(self, "Copy IOU template", "Template name:", QtWidgets.QLineEdit.Normal, "Copy of {}".format(copied_iou_device_settings["name"]))
            if ok:
                key = "{server}:{name}".format(server=copied_iou_device_settings["compute_id"], name=new_name)
                if key in self._iou_devices:
                    QtWidgets.QMessageBox.critical(self, "IOU template", "IOU template name {} already exists".format(new_name))
                    return
                self._iou_devices[key] = IOU_DEVICE_SETTINGS.copy()
                self._iou_devices[key].update(copied_iou_device_settings)
                self._iou_devices[key]["name"] = new_name
                self._iou_devices[key].pop("template_id", None)

                item = QtWidgets.QTreeWidgetItem(self.uiIOUDevicesTreeWidget)
                item.setText(0, self._iou_devices[key]["name"])
                Controller.instance().getSymbolIcon(self._iou_devices[key]["symbol"], qpartial(self._setItemIcon, item))
                item.setData(0, QtCore.Qt.UserRole, key)
                self._items.append(item)
                self.uiIOUDevicesTreeWidget.setCurrentItem(item)

    def _iouDeviceEditSlot(self):
        """
        Edits an IOU device.
        """

        item = self.uiIOUDevicesTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            iou_device = self._iou_devices[key]
            dialog = ConfigurationDialog(iou_device["name"], iou_device, iouDeviceConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                # update the icon
                Controller.instance().getSymbolIcon(iou_device["symbol"], qpartial(self._setItemIcon, item))
                if iou_device["name"] != item.text(0):
                    new_key = "{server}:{name}".format(server=iou_device["compute_id"], name=iou_device["name"])
                    if new_key in self._iou_devices:
                        QtWidgets.QMessageBox.critical(self, "IOU device", "IOU device name {} already exists for server {}".format(iou_device["name"],
                                                                                                                                    iou_device["compute_id"]))
                        iou_device["name"] = item.text(0)
                        return
                    self._iou_devices[new_key] = self._iou_devices[key]
                    del self._iou_devices[key]
                    item.setText(0, iou_device["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)
                self._refreshInfo(dialog.settings())

    def _iouDeviceDeleteSlot(self):
        """
        Deletes an IOU device.
        """

        for item in self.uiIOUDevicesTreeWidget.selectedItems():
            if item:
                key = item.data(0, QtCore.Qt.UserRole)
                del self._iou_devices[key]
                self.uiIOUDevicesTreeWidget.takeTopLevelItem(self.uiIOUDevicesTreeWidget.indexOfTopLevelItem(item))

    def loadPreferences(self):
        """
        Loads the IOU devices preferences.
        """

        self._iou_devices = {}
        templates = TemplateManager.instance().templates()
        for template_id, template in templates.items():
            if template.template_type() == "iou" and not template.builtin():
                name = template.name()
                server = template.compute_id()
                #TODO: use template id for the key
                key = "{server}:{name}".format(server=server, name=name)
                self._iou_devices[key] = copy.deepcopy(template.settings())

        self._items.clear()
        for key, iou_device in self._iou_devices.items():
            item = QtWidgets.QTreeWidgetItem(self.uiIOUDevicesTreeWidget)
            item.setText(0, iou_device["name"])
            Controller.instance().getSymbolIcon(iou_device["symbol"], qpartial(self._setItemIcon, item))

            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiIOUDevicesTreeWidget.setCurrentItem(self._items[0])
            self.uiIOUDevicesTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
            self.uiIOUDevicesTreeWidget.setMaximumWidth(self.uiIOUDevicesTreeWidget.sizeHintForColumn(0) + 10)

    def savePreferences(self):
        """
        Saves the IOU devices preferences.
        """

        templates = []
        for template in TemplateManager.instance().templates().values():
            if template.template_type() != "iou":
                templates.append(template)
        for template_settings in self._iou_devices.values():
            templates.append(Template(template_settings))
        TemplateManager.instance().updateList(templates)

    def _imageUploadComplete(self):
        if self._upload_image_progress_dialog.wasCanceled():
            return
        self._upload_image_progress_dialog.accept()

    @staticmethod
    def getImageDirectory():
        return ImageManager.instance().getDirectoryForType("IOU")

    @classmethod
    def getIOUImage(cls, parent, server):
        """

        :param parent: parent widget
        :param server: The server where the image is located

        :return: path to the IOU image or None
        """

        if not cls._default_images_dir:
            cls._default_images_dir = cls.getImageDirectory()

        path, _ = QtWidgets.QFileDialog.getOpenFileName(parent,
                                                        "Select an IOU image",
                                                        cls._default_images_dir,
                                                        "All file (*);;IOU image (*.bin *.image)",
                                                        "IOU image (*.bin *.image)")

        if not path:
            return
        cls._default_images_dir = os.path.dirname(path)

        if not os.access(path, os.R_OK):
            QtWidgets.QMessageBox.critical(parent, "IOU image", "Cannot read {}".format(path))
            return

        try:
            with open(path, "rb") as f:
                # read the first 7 bytes of the file.
                elf_header_start = f.read(7)
        except OSError as e:
            QtWidgets.QMessageBox.critical(parent, "IOU image", "Cannot read ELF magic number: {}".format(e))
            return

        # file must start with the ELF magic number, be 32-bit or 64-bit, little endian and have an ELF version of 1
        # (normal IOS image are big endian!)
        if elf_header_start != b'\x7fELF\x01\x01\x01' and elf_header_start != b'\x7fELF\x02\x01\x01':
            QtWidgets.QMessageBox.critical(parent, "IOU image", "Sorry, this is not a valid IOU image!")
            return

        try:
            os.makedirs(cls.getImageDirectory(), exist_ok=True)
        except OSError as e:
            QtWidgets.QMessageBox.critical(parent, "IOU images directory", "Could not create the IOU images directory {}: {}".format(cls.getImageDirectory(), e))
            return

        path = ImageManager.instance().askCopyUploadImage(parent, path, server, "IOU")

        if os.path.exists(path) and not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.warning(parent, "IOU image", "{} is not executable".format(path))

        return path

    def _setItemIcon(self, item, icon):
        item.setIcon(0, icon)
        self.uiIOUDevicesTreeWidget.setMaximumWidth(self.uiIOUDevicesTreeWidget.sizeHintForColumn(0) + 10)



#
# Copyright (C) 2023 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import atexit
import logging
import os
import sys

try:
    import importlib_resources
except ImportError:
    from importlib import resources as importlib_resources


from contextlib import ExitStack
resource_manager = ExitStack()
atexit.register(resource_manager.close)

log = logging.getLogger(__name__)


def get_resource(resource_name):
    """
    Return a resource in current directory or in frozen package
    """

    resource_path = None
    if hasattr(sys, "frozen"):
        resource_path = os.path.normpath(os.path.join(os.path.dirname(sys.executable), resource_name))
    else:
        ref = importlib_resources.files("gns3") / resource_name
        path = resource_manager.enter_context(importlib_resources.as_file(ref))
        if os.path.exists(path):
            resource_path = os.path.normpath(path)
    return resource_path
